@@ -17,22 +17,116 @@
 // DEALINGS IN THE SOFTWARE.
 
 use std;
-use python::{PythonObject, ToPythonPointer, Python, PythonObjectDowncastError, PythonObjectWithTypeObject, PyClone};
+use python::{PythonObject, ToPythonPointer, Python, PythonObjectDowncastError,
+        PythonObjectWithTypeObject, PyClone, PyDrop};
 use objects::{PyObject, PyType, exc};
 #[cfg(feature="python27-sys")]
 use objects::oldstyle::PyClass;
 use ffi;
 use libc;
+use std::ptr;
+use libc::c_char;
 use conversion::ToPyObject;
 use std::ffi::CString;
 
+/**
+Defines a new exception type.
+
+# Syntax
+`py_exception!(module, MyError)`
+
+* `module` is the name of the containing module.
+* `MyError` is the name of the new exception type.
+
+# Example
+```
+#[macro_use]
+extern crate cpython;
+
+use cpython::{Python, PyDict};
+
+py_exception!(mymodule, CustomError);
+
+fn main() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+    let ctx = PyDict::new(py);
+
+    ctx.set_item(py, "CustomError", py.get_type::<CustomError>()).unwrap();
+
+    py.run("assert str(CustomError) == \"<class 'mymodule.CustomError'>\"", None, Some(&ctx)).unwrap();
+    py.run("assert CustomError('oops').args == ('oops',)", None, Some(&ctx)).unwrap();
+}
+```
+*/
+#[macro_export]
+macro_rules! py_exception {
+    ($module: ident, $name: ident, $base: ty) => {
+        pub struct $name($crate::PyObject);
+
+        pyobject_newtype!($name);
+
+        impl $name {
+            pub fn new<'p, T: $crate::ToPyObject>(py: $crate::Python<'p>, args: T) -> $crate::PyErr {
+                $crate::PyErr::new::<$name, T>(py, args)
+            }
+        }
+
+        impl $crate::PythonObjectWithCheckedDowncast for $name {
+            #[inline]
+            fn downcast_from<'p>(py: $crate::Python<'p>, obj: $crate::PyObject)
+                -> Result<$name, $crate::PythonObjectDowncastError<'p>>
+            {
+                if <$name as $crate::PythonObjectWithTypeObject>::type_object(py).is_instance(py, &obj) {
+                    Ok(unsafe { $crate::PythonObject::unchecked_downcast_from(obj) })
+                } else {
+                    Err($crate::PythonObjectDowncastError(py))
+                }
+            }
+
+            #[inline]
+            fn downcast_borrow_from<'a, 'p>(py: $crate::Python<'p>, obj: &'a $crate::PyObject)
+                -> Result<&'a $name, $crate::PythonObjectDowncastError<'p>>
+            {
+                if <$name as $crate::PythonObjectWithTypeObject>::type_object(py).is_instance(py, obj) {
+                    Ok(unsafe { $crate::PythonObject::unchecked_downcast_borrow_from(obj) })
+                } else {
+                    Err($crate::PythonObjectDowncastError(py))
+                }
+            }
+        }
+
+        impl $crate::PythonObjectWithTypeObject for $name {
+            #[inline]
+            fn type_object(py: $crate::Python) -> $crate::PyType {
+                unsafe {
+                    static mut type_object: *mut $crate::_detail::ffi::PyTypeObject = 0 as *mut $crate::_detail::ffi::PyTypeObject;
+
+                    if type_object.is_null() {
+                        type_object = $crate::PyErr::new_type(
+                            py,
+                            concat!(stringify!($module), ".", stringify!($name)),
+                            Some($crate::PythonObject::into_object(py.get_type::<$base>())),
+                            None).as_type_ptr();
+                    }
+
+                    $crate::PyType::from_type_ptr(py, type_object)
+                }
+            }
+        }
+    };
+    ($module: ident, $name: ident) => {
+        py_exception!($module, $name, $crate::exc::Exception);
+    }
+}
+
 /// Represents a Python exception that was raised.
 #[derive(Debug)]
 pub struct PyErr {
     /// The type of the exception. This should be either a `PyClass` or a `PyType`.
     pub ptype : PyObject,
     /// The value of the exception.
-    /// 
+    ///
     /// This can be either an instance of `ptype`,
     /// a tuple of arguments to be passed to `ptype`'s constructor,
     /// or a single argument to be passed to `ptype`'s constructor.
@@ -53,6 +147,31 @@ impl PyErr {
         unsafe { !ffi::PyErr_Occurred().is_null() }
     }
 
+    /// Creates a new exception type with the given name, which must be of the form
+    /// `<module>.<ExceptionName>`, as required by `PyErr_NewException`.
+    ///
+    /// `base` can be an existing exception type to subclass, or a tuple of classes
+    /// `dict` specifies an optional dictionary of class variables and methods
+    pub fn new_type(py: Python, name: &str, base: Option<PyObject>, dict: Option<PyObject>) -> PyType {
+        let base: *mut ffi::PyObject = match base {
+            None => ptr::null_mut(),
+            Some(obj) => obj.steal_ptr()
+        };
+
+        let dict: *mut ffi::PyObject = match dict {
+            None => ptr::null_mut(),
+            Some(obj) => obj.steal_ptr()
+        };
+
+        unsafe {
+            let null_terminated_name = CString::new(name).unwrap();
+            let ptr: *mut ffi::PyObject = ffi::PyErr_NewException(null_terminated_name.as_ptr() as *mut c_char,
+                                                                  base,
+                                                                  dict);
+            PyObject::from_borrowed_ptr(py, ptr).unchecked_cast_into::<PyType>()
+        }
+    }
+
     /// Retrieves the current error from the Python interpreter's global state.
     /// The error is cleared from the Python interpreter.
     /// If no error is set, returns a `SystemError`.
@@ -161,9 +280,12 @@ impl PyErr {
     /// Return true if the current exception matches the exception in `exc`.
     /// If `exc` is a class object, this also returns `true` when `self` is an instance of a subclass.
     /// If `exc` is a tuple, all exceptions in the tuple (and recursively in subtuples) are searched for a match.
-    #[inline]
-    pub fn matches(&self, _py: Python, exc: &PyObject) -> bool {
-        unsafe { ffi::PyErr_GivenExceptionMatches(self.ptype.as_ptr(), exc.as_ptr()) != 0 }
+    pub fn matches<T>(&self, py: Python, exc: T) -> bool
+        where T: ToPyObject
+    {
+        exc.with_borrowed_ptr(py, |exc| unsafe {
+            ffi::PyErr_GivenExceptionMatches(self.ptype.as_ptr(), exc) != 0
+        })
     }
 
     /// Normalizes the error. This ensures that the exception value is an instance of the exception type.
@@ -199,7 +321,7 @@ impl PyErr {
             Err(_) =>
                 match self.ptype.cast_as::<PyClass>(py) {
                     Ok(_)  => py.get_type::<PyClass>(),
-                    Err(_) => py.None().get_type().clone_ref(py)
+                    Err(_) => py.None().get_type(py)
                 }
         }
     }
@@ -209,7 +331,7 @@ impl PyErr {
     pub fn get_type(&self, py: Python) -> PyType {
         match self.ptype.cast_as::<PyType>(py) {
             Ok(t)  => t.clone_ref(py),
-            Err(_) => py.None().get_type().clone_ref(py)
+            Err(_) => py.None().get_type(py)
         }
     }
 
@@ -244,6 +366,24 @@ impl PyErr {
     }
 }
 
+impl PyDrop for PyErr {
+    fn release_ref(self, py: Python) {
+        self.ptype.release_ref(py);
+        self.pvalue.release_ref(py);
+        self.ptraceback.release_ref(py);
+    }
+}
+
+impl PyClone for PyErr {
+    fn clone_ref(&self, py: Python) -> PyErr {
+        PyErr {
+            ptype: self.ptype.clone_ref(py),
+            pvalue: self.pvalue.clone_ref(py),
+            ptraceback: self.ptraceback.clone_ref(py)
+        }
+    }
+}
+
 /// Converts `PythonObjectDowncastError` to Python `TypeError`.
 impl <'p> std::convert::From<PythonObjectDowncastError<'p>> for PyErr {
     fn from(err: PythonObjectDowncastError<'p>) -> PyErr {
