@@ -19,10 +19,11 @@
 use std;
 use ffi;
 use libc::c_char;
-use python::{Python, PythonObject};
+use python::{Python, PythonObject, PyDrop};
 use objectprotocol::ObjectProtocol;
 use conversion::ToPyObject;
 use objects::{PyObject, PyTuple, PyDict, exc};
+use py_class::PythonObjectFromPyClassMacro;
 use err::{self, PyResult, PyErr};
 use std::ffi::{CStr, CString};
 
@@ -106,12 +107,16 @@ impl PyModule {
 
     /// Adds a new extension type to the module.
     ///
-    /// This is a convenience function that creates a new `PyRustTypeBuilder` and
-    /// sets `new_type.__module__` to this module's name.
-    /// The new type will be added to this module when `finish()` is called on the builder.
-    pub fn add_type<'p, T>(&self, py: Python<'p>, name: &str) -> ::rustobject::typebuilder::PyRustTypeBuilder<'p, T>
-            where T: 'static + Send {
-        ::rustobject::typebuilder::new_typebuilder_for_module(py, self, name)
+    /// This is a convenience function that initializes the `py_class!()`,
+    /// sets `new_type.__module__` to this module's name,
+    //  and adds the type to this module.
+    pub fn add_class<'p, T>(&self, py: Python<'p>) -> PyResult<()>
+        where T: PythonObjectFromPyClassMacro
+    {
+        let type_obj = try!(T::initialize(py));
+        try!(self.as_object().setattr(py, type_obj.name(py), &type_obj));
+        type_obj.release_ref(py);
+        Ok(())
     }
 }
 
