@@ -42,12 +42,14 @@ use err::PyResult;
 /// on `PyObject` to convert to more specific object types.
 ///
 /// Most of the interesting methods are provided by the [ObjectProtocol trait](trait.ObjectProtocol.html).
-#[unsafe_no_drop_flag]
 #[repr(C)]
 pub struct PyObject {
     // PyObject owns one reference to the *PyObject
     // ptr is not null
-    ptr: ptr::Shared<ffi::PyObject>
+    #[cfg(feature="nightly")]
+    ptr: ptr::Shared<ffi::PyObject>,
+    #[cfg(not(feature="nightly"))]
+    ptr: *mut ffi::PyObject,
 }
 
 // PyObject is thread-safe, because all operations on it require a Python<'p> token.
@@ -56,17 +58,35 @@ unsafe impl Sync for PyObject {}
 
 /// Dropping a `PyObject` decrements the reference count on the object by 1.
 impl Drop for PyObject {
-    #[inline]
     fn drop(&mut self) {
-        // TODO: remove `if` when #[unsafe_no_drop_flag] disappears
-        if *self.ptr as usize != mem::POST_DROP_USIZE {
-            let _gil_guard = Python::acquire_gil();
-            unsafe { ffi::Py_DECREF(*self.ptr); }
-        }
+        let _gil_guard = Python::acquire_gil();
+        unsafe { ffi::Py_DECREF(unpack_shared(self.ptr)); }
     }
 }
 
-pyobject_to_pyobject!(PyObject);
+#[inline]
+#[cfg(feature="nightly")]
+unsafe fn make_shared(ptr: *mut ffi::PyObject) -> ptr::Shared<ffi::PyObject> {
+    ptr::Shared::new(ptr)
+}
+
+#[inline]
+#[cfg(not(feature="nightly"))]
+unsafe fn make_shared(ptr: *mut ffi::PyObject) -> *mut ffi::PyObject {
+    ptr
+}
+
+#[inline]
+#[cfg(feature="nightly")]
+fn unpack_shared(ptr: ptr::Shared<ffi::PyObject>) -> *mut ffi::PyObject {
+    *ptr
+}
+
+#[inline]
+#[cfg(not(feature="nightly"))]
+fn unpack_shared(ptr: *mut ffi::PyObject) -> *mut ffi::PyObject {
+    ptr
+}
 
 impl PythonObject for PyObject {
     #[inline]
@@ -114,9 +134,9 @@ impl PyObject {
     /// This moves ownership over the pointer into the PyObject.
     /// Undefined behavior if the pointer is NULL or invalid.
     #[inline]
-    pub unsafe fn from_owned_ptr(_py : Python, ptr : *mut ffi::PyObject) -> PyObject {
+    pub unsafe fn from_owned_ptr(_py: Python, ptr: *mut ffi::PyObject) -> PyObject {
         debug_assert!(!ptr.is_null() && ffi::Py_REFCNT(ptr) > 0);
-        PyObject { ptr: ptr::Shared::new(ptr) }
+        PyObject { ptr: make_shared(ptr) }
     }
 
     /// Creates a PyObject instance for the given FFI pointer.
@@ -126,7 +146,7 @@ impl PyObject {
     pub unsafe fn from_borrowed_ptr(_py : Python, ptr : *mut ffi::PyObject) -> PyObject {
         debug_assert!(!ptr.is_null() && ffi::Py_REFCNT(ptr) > 0);
         ffi::Py_INCREF(ptr);
-        PyObject { ptr: ptr::Shared::new(ptr) }
+        PyObject { ptr: make_shared(ptr) }
     }
 
     /// Creates a PyObject instance for the given FFI pointer.
@@ -155,7 +175,7 @@ impl PyObject {
     /// Returns a borrowed pointer.
     #[inline]
     pub fn as_ptr(&self) -> *mut ffi::PyObject {
-        *self.ptr
+        unpack_shared(self.ptr)
     }
 
     /// Gets the underlying FFI pointer.
@@ -190,12 +210,9 @@ impl PyObject {
     }
 
     /// Gets the Python type object for this object's type.
-    #[inline]
-    pub fn get_type(&self) -> &PyType {
+    pub fn get_type(&self, py: Python) -> PyType {
         unsafe {
-            let t : &*mut ffi::PyTypeObject = &(*self.as_ptr()).ob_type;
-            let t : &*mut ffi::PyObject = mem::transmute(t);
-            PyObject::borrow_from_ptr(t).unchecked_cast_as()
+            PyType::from_type_ptr(py, (*self.as_ptr()).ob_type)
         }
     }
 
@@ -242,11 +259,10 @@ impl PyObject {
     /// Extracts some type from the Python object.
     /// This is a wrapper function around `FromPyObject::from_py_object()`.
     #[inline]
-    pub fn extract<T>(&self, py: Python) -> PyResult<T>
-        where T: for<'prep> ::conversion::ExtractPyObject<'prep>
+    pub fn extract<'a, T>(&'a self, py: Python) -> PyResult<T>
+        where T: ::conversion::FromPyObject<'a>
     {
-        let prepared = try!(<T as ::conversion::ExtractPyObject>::prepare_extract(py, self));
-        <T as ::conversion::ExtractPyObject>::extract(py, &prepared)
+        ::conversion::FromPyObject::extract(py, self)
     }
 }
 
