@@ -16,12 +16,12 @@
 // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-use python::{Python, PythonObject, ToPythonPointer};
+use python::{Python, PythonObject, ToPythonPointer, PyDrop};
 use err::{self, PyErr, PyResult};
 use super::object::PyObject;
 use super::exc;
 use ffi::{self, Py_ssize_t};
-use conversion::{ToPyObject, ExtractPyObject};
+use conversion::{FromPyObject, ToPyObject};
 use std::slice;
 
 /// Represents a Python tuple object.
@@ -72,7 +72,7 @@ impl PyTuple {
     }
 
     #[inline]
-    pub fn as_slice<'a>(&'a self) -> &'a [PyObject] {
+    pub fn as_slice<'a>(&'a self, py: Python) -> &'a [PyObject] {
         // This is safe because PyObject has the same memory layout as *mut ffi::PyObject,
         // and because tuples are immutable.
         // (We don't even need a Python token, thanks to immutability)
@@ -81,73 +81,14 @@ impl PyTuple {
             PyObject::borrow_from_owned_ptr_slice(
                 slice::from_raw_parts(
                     (*ptr).ob_item.as_ptr(),
-                    ffi::PyTuple_GET_SIZE(self.0.as_ptr()) as usize
+                    self.len(py)
                 ))
         }
     }
 
     #[inline]
-    pub fn iter<'a, 'p>(&'a self, py: Python<'p>) -> PyTupleIterator<'a, 'p> {
-        PyTupleIterator {
-            py: py,
-            tuple: self,
-            index: 0,
-            end: self.len(py)
-        }
-    }
-}
-
-impl ::std::ops::Index<usize> for PyTuple {
-    type Output = PyObject;
-
-    #[inline]
-    fn index(&self, index: usize) -> &PyObject {
-        &self.as_slice()[index]
-    }
-}
-
-impl <'a> IntoIterator for &'a PyTuple {
-    type Item = &'a PyObject;
-    type IntoIter = slice::Iter<'a, PyObject>;
-
-    #[inline]
-    fn into_iter(self) -> Self::IntoIter {
-        self.as_slice().iter()
-    }
-}
-
-/// Used by `PyTuple::iter()`.
-pub struct PyTupleIterator<'a, 'p> {
-    py: Python<'p>,
-    tuple: &'a PyTuple,
-    index: usize,
-    end: usize
-}
-
-impl <'a, 'p> Iterator for PyTupleIterator<'a, 'p> {
-    type Item = PyObject;
-
-    #[inline]
-    fn next(&mut self) -> Option<PyObject> {
-        if self.index < self.end {
-            let item = self.tuple.get_item(self.py, self.index);
-            self.index += 1;
-            Some(item)
-        } else {
-            None
-        }
-    }
-
-    #[inline]
-    fn size_hint(&self) -> (usize, Option<usize>) {
-        (self.len(), Some(self.len()))
-    }
-}
-
-impl <'a, 'p> ExactSizeIterator for PyTupleIterator<'a, 'p> {
-    #[inline]
-    fn len(&self) -> usize {
-        self.end - self.index
+    pub fn iter(&self, py: Python) -> slice::Iter<PyObject> {
+        self.as_slice(py).iter()
     }
 }
 
@@ -156,38 +97,37 @@ fn wrong_tuple_length(py: Python, t: &PyTuple, expected_length: usize) -> PyErr
     PyErr::new_lazy_init(py.get_type::<exc::ValueError>(), Some(msg.to_py_object(py).into_object()))
 }
 
-macro_rules! id (($a:expr) => ($a));
-
-macro_rules! tuple_conversion ({$length:expr,$(($refN:ident, $n:tt, $T:ident)),+} => (
-    impl <'p, $($T: ToPyObject),+> ToPyObject for ($($T,)+) {
+macro_rules! tuple_conversion ({$length:expr,$(($refN:ident, $n:tt, $T:ident)),+} => {
+    impl <$($T: ToPyObject),+> ToPyObject for ($($T,)+) {
         type ObjectType = PyTuple;
 
         fn to_py_object(&self, py: Python) -> PyTuple {
             PyTuple::new(py, &[
-                $(id!(self.$n.to_py_object(py)).into_object(),)+
+                $(py_coerce_expr!(self.$n.to_py_object(py)).into_object(),)+
             ])
         }
 
         fn into_py_object(self, py: Python) -> PyTuple {
             PyTuple::new(py, &[
-                $(id!(self.$n.into_py_object(py)).into_object(),)+
+                $(py_coerce_expr!(self.$n.into_py_object(py)).into_object(),)+
             ])
         }
     }
 
-    /*TODO: reimplement this without slice matching
-    impl <'p, 's, $($T: FromPyObject<'p, 's>),+> FromPyObject<'p, 's> for ($($T,)+) {
-        fn from_py_object(s : &'s PyObject<'p>) -> PyResult<'p, ($($T,)+)> {
-            let t = try!(s.cast_as::<PyTuple>());
-            match t.as_slice() {
-                [$(ref $refN,)+] => Ok((
-                    $(try!($refN.extract::<$T>()),)+
-                )),
-                _ => Err(wrong_tuple_length(t, 2))
+    impl <'s, $($T: FromPyObject<'s>),+> FromPyObject<'s> for ($($T,)+) {
+        fn extract(py: Python, obj: &'s PyObject) -> PyResult<Self> {
+            let t = try!(obj.cast_as::<PyTuple>(py));
+            let slice = t.as_slice(py);
+            if slice.len() == $length {
+                Ok((
+                    $( try!(slice[$n].extract::<$T>(py)), )+
+                ))
+            } else {
+                Err(wrong_tuple_length(py, t, $length))
             }
         }
-    }*/
-));
+    }
+});
 
 tuple_conversion!(1, (ref0, 0, A));
 tuple_conversion!(2, (ref0, 0, A), (ref1, 1, B));
@@ -252,6 +192,7 @@ mod test {
         let py = gil.python();
         let tuple = (1, 2, 3).to_py_object(py);
         assert_eq!(3, tuple.len(py));
+        assert_eq!((1, 2, 3), tuple.into_object().extract(py).unwrap());
     }
 }
 
